@@ -21,8 +21,8 @@
 
 hkda <- function(x, x.group, bw="plugin", nstage=2, binned=TRUE, bgridsize)
 {
-  grlab <- sort(unique(x.group))
-  m <- length(grlab)
+  gr <- sort(unique(x.group))
+  m <- length(gr)
   bw <- substr(tolower(bw),1,1)
   hs <- numeric(0)
 
@@ -30,7 +30,7 @@ hkda <- function(x, x.group, bw="plugin", nstage=2, binned=TRUE, bgridsize)
   
   for (i in 1:m)
   {
-    y <- x[x.group==grlab[i]]
+    y <- x[x.group==gr[i]]
     if (bw=="p")
       h <- hpi(y, nstage=nstage, binned=TRUE, bgridsize=bgridsize) 
     hs <- c(hs, h)
@@ -42,8 +42,8 @@ hkda <- function(x, x.group, bw="plugin", nstage=2, binned=TRUE, bgridsize)
 Hkda <- function(x, x.group, Hstart, bw="plugin", nstage=2, pilot="samse", pre="sphere", binned=FALSE, bgridsize)
 {
   d <- ncol(x)
-  grlab <- sort(unique(x.group))
-  m <- length(grlab)
+  gr <- sort(unique(x.group))
+  m <- length(gr)
   bw <- substr(tolower(bw),1,1)
   Hs <- numeric(0)
 
@@ -51,7 +51,7 @@ Hkda <- function(x, x.group, Hstart, bw="plugin", nstage=2, pilot="samse", pre="
   
   for (i in 1:m)
   {
-    y <- x[x.group==grlab[i],]
+    y <- x[x.group==gr[i],]
     if (!missing(Hstart)) 
     {
       Hstarty <- Hstart[((i-1)*d+1) : (i*d),]
@@ -74,8 +74,8 @@ Hkda <- function(x, x.group, Hstart, bw="plugin", nstage=2, pilot="samse", pre="
 Hkda.diag <- function(x, x.group, bw="plugin", nstage=2, pilot="samse", pre="sphere", binned=FALSE, bgridsize)
 {
   d <- ncol(x)
-  grlab <- sort(unique(x.group))
-  m <- length(grlab)
+  gr <- sort(unique(x.group))
+  m <- length(gr)
   bw <- substr(tolower(bw),1,1)
   Hs <- numeric(0)
 
@@ -83,7 +83,7 @@ Hkda.diag <- function(x, x.group, bw="plugin", nstage=2, pilot="samse", pre="sph
 
   for (i in 1:m)
   {
-    y <- x[x.group==grlab[i],]
+    y <- x[x.group==gr[i],]
     if (bw=="l") H <- Hlscv.diag(y, binned=binned, bgridsize=bgridsize)
     else if (bw=="p") H <- Hpi.diag(y, nstage=nstage, pilot=pilot, pre=pre, binned=binned, bgridsize=bgridsize)
     else if (bw=="s") H <- Hscv.diag(y, pre=pre, binned=binned, bgridsize=bgridsize)
@@ -95,107 +95,6 @@ Hkda.diag <- function(x, x.group, bw="plugin", nstage=2, pilot="samse", pre="sph
 
 
 
-###############################################################################
-# Classify data set according to discriminant analysis based on training data
-# for 1- to 6-dim
-#
-# Parameter
-# x - training data
-# x.group - group variable for x
-# y - data values to be classified
-# Hs - bandwidth matrices
-# prior.prob - prior probabilities
-#
-# Returns
-# Group classification of data set y
-###############################################################################
-
-kda <- function(x, x.group, Hs, hs, y, prior.prob=NULL)
-{
-  if (missing(y)) y <- x
-  if (is.vector(x))
-  {
-    disc.gr <- kda.1d(x=x, x.group=x.group, hs=hs, y=y, prior.prob=prior.prob)
-  }
-  else
-  {  
-    if (is.data.frame(x)) x <- as.matrix(x)
-    if (is.data.frame(y)) y <- as.matrix(y)
-    gr <- sort(unique(x.group))
-
-    ##d <- ncol(x)
-   
-    ## if prior.prob is NULL then use sample proportions
-    if (is.null(prior.prob))
-    {
-      prior.prob <- rep(0, length(gr))
-      for (j in 1:length(gr))
-        prior.prob[j] <- length(which(x.group==gr[j]))
-      prior.prob <- prior.prob/nrow(x)
-    }
-    
-    if (!(identical(all.equal(sum(prior.prob), 1), TRUE)))  
-      stop("Sum of prior weights not equal to 1")
-    
-    ## Compute KDE and weighted KDE 
-    m <- length(gr)
-    fhat <- kda.kde(x, x.group, Hs=Hs, eval.points=y)
-    fhat.wt <- matrix(0, ncol=m, nrow=nrow(y))  
-    
-    for (j in 1:m)
-      fhat.wt[,j] <- fhat$est[[j]]* prior.prob[j]
-    
-    ## Assign y according largest weighted density value 
-    disc.gr.temp <- apply(fhat.wt, 1, which.max)
-    
-    disc.gr <- gr
-    for (j in 1:m)
-    {
-      ind <- which(disc.gr.temp==j)
-      disc.gr[ind] <- gr[j]
-    }
-  }
-  
-  return(disc.gr) 
-}
-
-kda.1d <- function(x, x.group, hs, y, prior.prob=NULL)
-{ 
-  gr <- sort(unique(x.group))
-
-  # if prior.prob is NULL then use sample proportions
-  if (is.null(prior.prob))
-  {
-    prior.prob <- rep(0, length(gr))
-    for (j in 1:length(gr))
-      prior.prob[j] <- length(which(x.group==gr[j]))
-    prior.prob <- prior.prob/length(x)
-  }
-  
-  if (!(identical(all.equal(sum(prior.prob), 1), TRUE)))  
-    stop("Sum of prior weights not equal to 1")
-
-  ## Compute KDE and weighted KDE 
-  m <- length(gr)
-  fhat <- kda.kde(x, x.group, hs=hs, eval.points=y)
-  fhat.wt <- matrix(0, ncol=m, nrow=length(y))  
-  
-  for (j in 1:m)
-    fhat.wt[,j] <- fhat$estimate[[j]]* prior.prob[j]
-
-  ## Assign y according largest weighted density value 
-  disc.gr.temp <- apply(fhat.wt, 1, which.max)
-
-  disc.gr <- gr
-  for (j in 1:m)
-  {
-    ind <- which(disc.gr.temp==j)
-    disc.gr[ind] <- gr[j]
-  }
- 
-  return(disc.gr) 
-}
-
 
 ###############################################################################
 # Compares true group classification with an estimated one
@@ -214,7 +113,7 @@ kda.1d <- function(x, x.group, hs, y, prior.prob=NULL)
 compare <- function(x.group, est.group, by.group=FALSE)
 {
   if (length(x.group)!=length(est.group))
-    stop("Group label vectors not the same length")
+    stop("group label vectors not the same length")
  
   grlab <- sort(unique(x.group))
   m <- length(grlab)
@@ -266,7 +165,7 @@ compare <- function(x.group, est.group, by.group=FALSE)
 # error - total mis-classification rate
 ###############################################################################
 
-compare.kda.cv <- function(x, x.group, bw="plugin", prior.prob=NULL, Hstart, by.group=FALSE, verbose=FALSE, trace=verbose, binned=FALSE, bgridsize, recompute=FALSE, ...)
+compare.kda.cv <- function(x, x.group, bw="plugin", prior.prob=NULL, Hstart, by.group=FALSE, verbose=FALSE, binned=FALSE, bgridsize, recompute=FALSE, ...)
 {
   if (verbose) pb <- txtProgressBar()
 
@@ -293,8 +192,8 @@ compare.kda.cv <- function(x, x.group, bw="plugin", prior.prob=NULL, Hstart, by.
     
       ## recompute KDA estimate of groups with x[i] excluded
       
-      if (verbose) setTxtProgressBar(pb, i/n)  
-      kda.cv.gr[i] <- kda(x[-i], x.group[-i], hs=h.mod, y=x, prior.prob=prior.prob)[i]
+      if (verbose) setTxtProgressBar(pb, i/n)
+      kda.cv.gr[i] <- kda(x[-i], x.group[-i], hs=h.mod, eval.points=x, prior.prob=prior.prob, kde.flag=FALSE)[i]
     }
     if (verbose) close(pb)
     return(compare(x.group, kda.cv.gr, by.group=by.group)) 
@@ -355,7 +254,7 @@ compare.kda.cv <- function(x, x.group, bw="plugin", prior.prob=NULL, Hstart, by.
       
     if (verbose) setTxtProgressBar(pb, i/n)  ##cat(paste("Processing data item:", i, "\n"))
 
-    kda.cv.gr[i] <- kda(x[-i,], x.group[-i], Hs=H.mod, y=x, prior.prob=prior.prob)[i]
+    kda.cv.gr[i] <- kda(x[-i,], x.group[-i], Hs=H.mod, eval.points=x, prior.prob=prior.prob, kde.flag=FALSE)[i]
   }
   if (verbose) close(pb)
 
@@ -367,7 +266,7 @@ compare.kda.cv <- function(x, x.group, bw="plugin", prior.prob=NULL, Hstart, by.
 ###############################################################################
 
 compare.kda.diag.cv <- function(x, x.group, bw="plugin", prior.prob=NULL,
-   by.group=FALSE, verbose=FALSE, trace=verbose, binned=FALSE, bgridsize, recompute=FALSE, ...)
+   by.group=FALSE, verbose=FALSE, binned=FALSE, bgridsize, recompute=FALSE, ...)
 {
   if (is.vector(x))  return(compare.kda.cv(x=x, x.group=x.group, by.group=by.group, verbose=verbose, binned=binned, bgridsize=bgridsize, recompute=recompute, ...))
   n <- nrow(x)
@@ -399,7 +298,7 @@ compare.kda.diag.cv <- function(x, x.group, bw="plugin", prior.prob=NULL,
     }
     
     if (verbose) setTxtProgressBar(pb, i/n) 
-    kda.cv.gr[i] <- kda(x[-i,], x.group[-i], Hs=H.mod, y=x, prior.prob=prior.prob)[i]  
+    kda.cv.gr[i] <- kda(x[-i,], x.group[-i], Hs=H.mod, eval.points=x, prior.prob=prior.prob, kde.flag=FALSE)[i]  
   }
   if (verbose) close(pb)
   
@@ -408,6 +307,7 @@ compare.kda.diag.cv <- function(x, x.group, bw="plugin", prior.prob=NULL,
 
 
 
+
 ###############################################################################
 # KDEs of individual densities for KDA - 1- to 3-dim
 #
@@ -424,102 +324,76 @@ compare.kda.diag.cv <- function(x, x.group, bw="plugin", prior.prob=NULL,
 # H - list of bandwidth matrices
 ##############################################################################
 
-kda.kde <- function(x, x.group, Hs, hs, prior.prob=NULL, gridsize, xmin, xmax, supp=3.7, eval.points=NULL, binned=FALSE, bgridsize, w, compute.cont=FALSE, approx.cont=TRUE)
+kda <- function(x, x.group, Hs, hs, prior.prob=NULL, gridsize, xmin, xmax, supp=3.7, eval.points, binned=FALSE, bgridsize, w, compute.cont=FALSE, approx.cont=TRUE, kde.flag=TRUE)
 {
+  if (missing(eval.points)) eval.points <- x
+  gr <- sort(unique(x.group))
+  m <- length(gr)
+
   if (is.vector(x))
   {
-    if (missing(gridsize))  gridsize <- 101
-    if (missing(bgridsize)) bgridsize <- 401
-    fhat.list <- kda.kde.1d(x=x, x.group=x.group, hs=hs, prior.prob=prior.prob, gridsize=gridsize, eval.points=eval.points, supp=supp, binned=binned, bgridsize=bgridsize, xmin=xmin, xmax=xmax, compute.cont=compute.cont, approx.cont=approx.cont)
+    if (missing(hs)) hs <- hkda(x=x, x.group=x.group, bw="plugin", nstage=2, binned=TRUE, bgridsize=bgridsize)
+    ## Compute KDA on grid
+    if (kde.flag)
+      fhat.list <- kda.1d(x=x, x.group=x.group, hs=hs, prior.prob=prior.prob, gridsize=gridsize, supp=supp, binned=binned, bgridsize=bgridsize, xmin=xmin, xmax=xmax, compute.cont=compute.cont, approx.cont=approx.cont)
+    
+    ## Compute KDA at eval.points
+    fhat <- kda.1d(x=x, x.group=x.group, hs=hs, prior.prob=prior.prob, gridsize=gridsize, supp=supp, binned=FALSE, bgridsize=bgridsize, xmin=xmin, xmax=xmax, eval.points=eval.points, compute.cont=compute.cont, approx.cont=approx.cont)
+    fhat.wt <- matrix(0, ncol=m, nrow=length(eval.points))  
   }
   else
   {
-    if (is.data.frame(x)) x <- as.matrix(x)
-    grlab <- sort(unique(x.group))
-    m <- length(grlab)
-    d <- ncol(x)
-
-    ## find largest bandwidth matrix to initialise grid
-    detH <- vector() 
-    for (j in 1:m)
-      detH[j] <- det(Hs[((j-1)*d+1) : (j*d),])  
-    Hmax.ind <- which.max(detH)
-    Hmax <- Hs[((Hmax.ind-1)*d+1) : (Hmax.ind*d),]
-
-    if (missing(xmin)) xmin <- apply(x, 2, min) - supp*4*max(sqrt(diag(Hmax)))
-    if (missing(xmax)) xmax <- apply(x, 2, max) + supp*4*max(sqrt(diag(Hmax)))
-    if (missing(w)) w <- rep(1, nrow(x))
+    if (ncol(x)==2) pilot <- "samse"
+    if (ncol(x)==3) pilot <- "dscalar"
+    if (missing(Hs)) Hs <- Hkda(x=x, x.group=x.group, bw="plugin", nstage=2, pilot=pilot, pre="sphere", binned=nrow(x)>1000, bgridsize=bgridsize)
+  
+    if (kde.flag)
+      fhat.list <- kda.nd(x=x, x.group=x.group, Hs=Hs, prior.prob=prior.prob, gridsize=gridsize, supp=supp, binned=binned, bgridsize=bgridsize, xmin=xmin, xmax=xmax, compute.cont=compute.cont, approx.cont=approx.cont)
     
-    if (binned & d > 4)
-      stop("Binning only available for 1- to 4-dim data")
-    
-    if (missing(bgridsize)) bgridsize <- default.gridsize(d)
-    if (missing(gridsize)) gridsize <- default.gridsize(d)
-       
-    fhat.list <- list()
-    for (j in 1:m)
-    {
-      xx <- x[x.group==grlab[j],]
-      ww <- w[x.group==grlab[j]]   
-      H <- Hs[((j-1)*d+1) : (j*d),]     
-      
-      ## compute individual density estimate
-      if (binned)
-        fhat.temp <- kdde.binned(x=xx, bgridsize=bgridsize, H=H, xmin=xmin, xmax=xmax, w=ww, deriv.order=0)
-      else if (is.null(eval.points))
-        fhat.temp <- kde(x=xx, H=H, supp=supp, xmin=xmin, xmax=xmax, gridsize=gridsize, w=ww)
-      else
-        fhat.temp <- kde(x=xx, H=H, eval.points=eval.points, w=ww)
-      
-      fhat.list$estimate <- c(fhat.list$estimate, list(fhat.temp$estimate))
-      fhat.list$eval.points <- fhat.temp$eval.points
-      fhat.list$x <- c(fhat.list$x, list(xx))
-      fhat.list$H <- c(fhat.list$H, list(H))
-      fhat.list$w <- c(fhat.list$w, list(ww))
-      
-      ## compute prob contour levels
-      if (compute.cont & missing(eval.points))
-      {
-        contlev <- contourLevels(fhat.temp, cont=1:99, approx.cont=approx.cont)
-        fhat.list$cont <- c(fhat.list$cont, list(contlev))
-      }
-    }
-    
-    fhat.list$binned <- binned
-    fhat.list$gridded <- fhat.temp$gridded
-    fhat.list$x.group <- x.group
-    pr <- rep(0, length(grlab))
-    for (j in 1:length(grlab)) pr[j] <- length(which(x.group==grlab[j]))
-    pr <- pr/nrow(x)
-    fhat.list$prior.prob <- pr
-    
-    class(fhat.list) <- "kda.kde"
+    fhat <- kda.nd(x=x, x.group=x.group, Hs=Hs, prior.prob=prior.prob, gridsize=gridsize, supp=supp, binned=binned, bgridsize=bgridsize, xmin=xmin, xmax=xmax, eval.points=eval.points, compute.cont=compute.cont, approx.cont=approx.cont)
+    fhat.wt <- matrix(0, ncol=m, nrow=nrow(eval.points))  
   }
+
+  for (j in 1:m)
+    fhat.wt[,j] <- fhat$estimate[[j]]* fhat$prior.prob[j]
+    
+  ## Assign y according largest weighted density value 
+  disc.gr.temp <- apply(fhat.wt, 1, which.max)
+  
+  disc.gr <- gr
+  for (j in 1:m)
+  {
+    ind <- which(disc.gr.temp==j)
+    disc.gr[ind] <- gr[j]
+  }
+  if (kde.flag) fhat.list$x.group.estimate <- disc.gr
+  else fhat.list <- disc.gr
   
   return(fhat.list)
 }
 
-kda.kde.1d <- function(x, x.group, hs, prior.prob, gridsize, supp, eval.points, binned, bgridsize, xmin, xmax, w, compute.cont, approx.cont)
+kda.1d <- function(x, x.group, hs, prior.prob, gridsize, supp, eval.points, binned, bgridsize, xmin, xmax, w, compute.cont, approx.cont)
 {
-  grlab <- sort(unique(x.group))
-  m <- length(grlab)
-
+  gr <- sort(unique(x.group))
+  m <- length(gr)
+  d <- 1
   hmax <- max(hs)
   if (missing(xmin)) xmin <- min(x) - supp*hmax
   if (missing(xmax)) xmax <- max(x) + supp*hmax
   if (missing(w)) w <- rep(1, length(x))
-  
+  if (missing(bgridsize)) bgridsize <- default.gridsize(d)
+  if (missing(gridsize)) gridsize <- default.gridsize(d)  
   fhat.list <- list()
   for (j in 1:m)
   {
-    xx <- x[x.group==grlab[j]]
-    ww <- w[x.group==grlab[j]]
+    xx <- x[x.group==gr[j]]
+    ww <- w[x.group==gr[j]]
     h <- hs[j]
     
     ## compute individual density estimate
     if (binned)
       fhat.temp <- kdde.binned(x=xx, h=h, xmin=xmin, xmax=xmax, bgridsize=bgridsize, w=ww, deriv.order=0)
-    else if (is.null(eval.points))
+    else if (missing(eval.points))
       fhat.temp <- kde(x=xx, h=h, supp=supp, xmin=xmin, xmax=xmax, gridsize=gridsize, w=ww)
     else
       fhat.temp <- kde(x=xx, h=h, eval.points=eval.points, w=ww)
@@ -528,7 +402,9 @@ kda.kde.1d <- function(x, x.group, hs, prior.prob, gridsize, supp, eval.points,
     fhat.list$eval.points <- fhat.temp$eval.points
     fhat.list$x <- c(fhat.list$x, list(xx))
     fhat.list$h <- c(fhat.list$h, h)
-
+    fhat.list$H <- c(fhat.list$H, h^2)
+    fhat.list$w <- c(fhat.list$w, list(ww))
+    
     ## compute prob contour levels
     if (compute.cont & missing(eval.points))
     {
@@ -536,34 +412,98 @@ kda.kde.1d <- function(x, x.group, hs, prior.prob, gridsize, supp, eval.points,
       fhat.list$cont <- c(fhat.list$cont, list(contlev))
     }
   }
-    
-  fhat.list$H <- fhat.list$h^2
-  fhat.list$w <- c(fhat.list$w, list(ww))
+
   fhat.list$binned <- binned
   fhat.list$gridded <- fhat.temp$gridded
-  fhat.list$x.group <- x.group
   
   if (is.null(prior.prob))
   {
-    pr <- rep(0, length(grlab))
-    for (j in 1:length(grlab))
-      pr[j] <- length(which(x.group==grlab[j]))
+    pr <- rep(0, length(gr))
+    for (j in 1:length(gr))
+      pr[j] <- length(which(x.group==gr[j]))
     pr <- pr/length(x)
     fhat.list$prior.prob <- pr
   }
   else
     fhat.list$prior.prob <- prior.prob
- 
-  class(fhat.list) <- "kda.kde"
-  
-  return(fhat.list)
+
+  fhat.list$x.group <- x.group
+  class(fhat.list) <- "kda"
   
+  return(fhat.list) 
 }
+
+
+kda.nd <- function(x, x.group, Hs, prior.prob, gridsize, supp, eval.points, binned, bgridsize, xmin, xmax, w, compute.cont, approx.cont)
+{
+  if (is.data.frame(x)) x <- as.matrix(x)
+  gr <- sort(unique(x.group))
+  m <- length(gr)
+  d <- ncol(x)
+  
+  ## find largest bandwidth matrix to initialise grid
+  detH <- vector() 
+  for (j in 1:m)
+      detH[j] <- det(Hs[((j-1)*d+1) : (j*d),])  
+  Hmax.ind <- which.max(detH)
+  Hmax <- Hs[((Hmax.ind-1)*d+1) : (Hmax.ind*d),]
+  
+  if (missing(xmin)) xmin <- apply(x, 2, min) - supp*4*max(sqrt(diag(Hmax)))
+  if (missing(xmax)) xmax <- apply(x, 2, max) + supp*4*max(sqrt(diag(Hmax)))
+  if (missing(w)) w <- rep(1, nrow(x))
+  
+  if (binned & d > 4) stop("binning only available for 1- to 4-d data")
+  if (missing(bgridsize)) bgridsize <- default.gridsize(d)
+  if (missing(gridsize)) gridsize <- default.gridsize(d)
+  
+  fhat.list <- list()
+  for (j in 1:m)
+  {
+    xx <- x[x.group==gr[j],]
+    ww <- w[x.group==gr[j]]   
+    H <- Hs[((j-1)*d+1) : (j*d),]     
+    
+    ## compute individual density estimate
+    if (binned)
+      fhat.temp <- kdde.binned(x=xx, bgridsize=bgridsize, H=H, xmin=xmin, xmax=xmax, w=ww, deriv.order=0)
+    else if (missing(eval.points))
+      fhat.temp <- kde(x=xx, H=H, supp=supp, xmin=xmin, xmax=xmax, gridsize=gridsize, w=ww)
+    else
+      fhat.temp <- kde(x=xx, H=H, eval.points=eval.points, w=ww)
+    
+    fhat.list$estimate <- c(fhat.list$estimate, list(fhat.temp$estimate))
+    fhat.list$eval.points <- fhat.temp$eval.points
+    fhat.list$x <- c(fhat.list$x, list(xx))
+    fhat.list$H <- c(fhat.list$H, list(H))
+    fhat.list$w <- c(fhat.list$w, list(ww))
+    
+      ## compute prob contour levels
+    if (compute.cont & missing(eval.points))
+    {
+      contlev <- contourLevels(fhat.temp, cont=1:99, approx.cont=approx.cont)
+      fhat.list$cont <- c(fhat.list$cont, list(contlev))
+    }
+  }
+  
+  fhat.list$binned <- binned
+  fhat.list$gridded <- fhat.temp$gridded
+  pr <- rep(0, length(gr))
+  for (j in 1:length(gr)) pr[j] <- length(which(x.group==gr[j]))
+  pr <- pr/nrow(x)
+  fhat.list$prior.prob <- pr
+  fhat.list$x.group <- x.group
+  
+  class(fhat.list) <- "kda"
+  return (fhat.list)
+}
+
+
+  
 ##############################################################################
 ## Contour method for kda.kde cobjects
 ##############################################################################
 
-contourLevels.kda.kde <- function(x, prob, cont, nlevels=5, approx=FALSE,...) 
+contourLevels.kda <- function(x, prob, cont, nlevels=5, approx=FALSE,...) 
 {
   fhat <- x
   m <- length(fhat$x)
@@ -595,23 +535,23 @@ contourLevels.kda.kde <- function(x, prob, cont, nlevels=5, approx=FALSE,...)
 ##############################################################################
 
 
-plot.kda.kde <- function(x, y, y.group, ...) 
+plot.kda <- function(x, y, y.group, ...) 
 {
   if (is.vector(x$x[[1]]))
-    plotkda.kde.1d(x=x, y=y, y.group=y.group,  ...)
+    plotkda.1d(x=x, y=y, y.group=y.group,  ...)
   else
   {  
     d <- ncol(x$x[[1]])
     
     if (d==2)
-      plotkda.kde.2d(x=x, y=y, y.group=y.group, ...) 
+      plotkda.2d(x=x, y=y, y.group=y.group, ...) 
     else if (d==3)  
-       plotkda.kde.3d(x=x, y=y, y.group=y.group, ...) 
+       plotkda.3d(x=x, y=y, y.group=y.group, ...) 
   }
 }
 
 
-plotkda.kde.1d <- function(x, y, y.group, prior.prob=NULL, xlim, ylim, xlab="x", ylab="Weighted density function", drawpoints=FALSE, col, partcol, ptcol, lty, jitter=TRUE, rugsize, ...)
+plotkda.1d <- function(x, y, y.group, prior.prob=NULL, xlim, ylim, xlab="x", ylab="Weighted density function", drawpoints=FALSE, col, partcol, ptcol, lty, jitter=TRUE, rugsize, ...)
 { 
   fhat <- x
   
@@ -622,13 +562,12 @@ plotkda.kde.1d <- function(x, y, y.group, prior.prob=NULL, xlim, ylim, xlab="x",
     prior.prob <- fhat$prior.prob
   
   if (m != length(prior.prob))
-    stop("Prior prob. vector not same length as number of components in fhat")
+    stop("prior prob. vector not same length as number of components in fhat")
   if (!(identical(all.equal(sum(prior.prob), 1), TRUE)))  
-    stop("Sum of prior weights not equal to 1")
+    stop("sum of weights not equal to 1")
 
   weighted.fhat <- matrix(0, nrow=length(fhat$eval.points), ncol=m) 
-  for (j in 1:m)
-    weighted.fhat[,j] <- fhat$estimate[[j]]*fhat$prior.prob[j]
+  for (j in 1:m)  weighted.fhat[,j] <- fhat$estimate[[j]]*fhat$prior.prob[j]
   
   if (missing(xlim)) xlim <- range(fhat$eval.points)
   if (missing(ylim)) ylim <- range(weighted.fhat)
@@ -651,7 +590,7 @@ plotkda.kde.1d <- function(x, y, y.group, prior.prob=NULL, xlim, ylim, xlab="x",
   ##eval.points.gr <- apply(weighted.fhat, 1, which.max)
 
   ydata <- seq(min(fhat$eval.points), max(fhat$eval.points), length=401)
-  ydata.gr <- kda(unlist(fhat$x), x.group=fhat$x.group, hs=fhat$h, y=ydata, prior.prob=fhat$prior.prob)
+  ydata.gr <- kda(unlist(fhat$x), binned=FALSE, x.group=fhat$x.group, hs=fhat$h, eval.points=ydata, prior.prob=fhat$prior.prob)$x.group.estimate
 
   ## draw partition class as rug-like plot
  
@@ -660,8 +599,6 @@ plotkda.kde.1d <- function(x, y, y.group, prior.prob=NULL, xlim, ylim, xlab="x",
 
   for (j in 1:m)
   {
-    ##rug(ydata[ydata.gr==levels(fhat$x.group)[j]], col=partcol[j])
-    ##image(ydata, c(plot.lim[3], plot.lim[3]+rugsize), cbind(ydata.gr==levels(fhat$x.group)[j], ydata.gr==levels(fhat$x.group)[j]), level=0.5, add=TRUE, col=c("transparent", partcol[j]), ...)
     image(ydata, c(plot.lim[3], plot.lim[3]+rugsize), cbind(as.numeric(ydata.gr), as.numeric(ydata.gr)), level=0.5+(0:length(levels(fhat$x.group))), add=TRUE, col=partcol, ...)
   } 
   
@@ -693,7 +630,7 @@ plotkda.kde.1d <- function(x, y, y.group, prior.prob=NULL, xlim, ylim, xlab="x",
 }
 
 
-plotkda.kde.2d <- function(x, y, y.group, prior.prob=NULL, 
+plotkda.2d <- function(x, y, y.group, prior.prob=NULL, 
     cont=c(25,50,75), abs.cont, approx.cont=FALSE, xlim, ylim, xlab, ylab,
     drawpoints=FALSE, drawlabels=TRUE, cex=1, pch, lty, col, partcol, ptcol, ...)
 { 
@@ -739,9 +676,9 @@ plotkda.kde.2d <- function(x, y, y.group, prior.prob=NULL,
     prior.prob <- fhat$prior.prob
 
   if (m != length(prior.prob))
-    stop("Prior prob. vector not same length as number of components in fhat")
+    stop("prior prob. vector not same length as number of components in fhat")
   if (!(identical(all.equal(sum(prior.prob), 1), TRUE)))  
-    stop("Sum of prior weights not equal to 1")
+    stop("sum of weights not equal to 1")
 
   ## set up plot
   if (missing(y)) 
@@ -822,11 +759,8 @@ plotkda.kde.2d <- function(x, y, y.group, prior.prob=NULL,
 
 
 
-plotkda.kde.3d <- function(x, y, y.group, prior.prob=NULL, cont=c(25,50,75), abs.cont, approx.cont=FALSE, colors, alphavec, xlab, ylab, zlab, drawpoints=FALSE, size=3, ptcol="blue", ...)
+plotkda.3d <- function(x, y, y.group, prior.prob=NULL, cont=c(25,50,75), abs.cont, approx.cont=FALSE, colors, alphavec, xlab, ylab, zlab, drawpoints=FALSE, size=3, ptcol="blue", ...)
 {
-  require(rgl)
-  require(misc3d)
-  
   fhat <- x
    
   ##d <- 3
@@ -834,18 +768,15 @@ plotkda.kde.3d <- function(x, y, y.group, prior.prob=NULL, cont=c(25,50,75), abs
   if (is.null(prior.prob))
     prior.prob <- fhat$prior.prob
   if (m != length(prior.prob))
-    stop("Prior prob. vector not same length as number of components in fhat")
+    stop("prior prob. vector not same length as number of components in fhat")
   if (!(identical(all.equal(sum(prior.prob), 1), TRUE)))  
-    stop("Sum of prior weights not equal to 1")
+    stop("sum of prior weights not equal to 1")
 
   x.names <- colnames(fhat$x[[1]])
 
-  if (missing(xlab))    
-    if (is.null(x.names)) xlab <- "x" else xlab <- x.names[1]
-  if (missing(ylab))
-    if (is.null(x.names)) ylab <- "y" else ylab <- x.names[2]
-  if (missing(zlab))
-    if (is.null(x.names)) zlab <- "z" else zlab <- x.names[3]
+  if (missing(xlab)) if (is.null(x.names)) xlab <- "x" else xlab <- x.names[1]
+  if (missing(ylab)) if (is.null(x.names)) ylab <- "y" else ylab <- x.names[2]
+  if (missing(zlab)) if (is.null(x.names)) zlab <- "z" else zlab <- x.names[3]
              
   xx <- numeric(0)
   for (j in 1:m) xx <- rbind(xx, fhat$x[[j]])
@@ -872,7 +803,7 @@ plotkda.kde.3d <- function(x, y, y.group, prior.prob=NULL, cont=c(25,50,75), abs
   if (length(ptcol)==1) ptcol <- rep(ptcol, m)
   
   ##clear3d()
-  plot3d(x=fhat$eval.points[[1]], y=fhat$eval.points[[2]], z=fhat$eval.points[[3]], type="n", xlab=xlab, ylab=ylab, zlab=zlab, ...)
+ plot3d(x=fhat$x[[1]][,1], y=fhat$x[[1]][,2], z=fhat$x[[1]][,3], type="n", xlab=xlab, ylab=ylab, zlab=zlab, ...)
   
   for (j in 1:m)
   {
